#!/usr/bin/python
#
# Copyright (c) 2009 Christophe Varoqui <christophe.varoqui@free.fr>'
# Copyright (c) 2009 Cyril Galibern <cyril.galibern@free.fr>'
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
from datetime import datetime
import xmlrpclib
import os
from rcGlobalEnv import rcEnv

sysname, nodename, x, x, machine = os.uname()
hostId = __import__('hostid'+sysname)
hostid = hostId.hostid()
proxy = xmlrpclib.ServerProxy(rcEnv.dbopensvc)

def begin_action(svc, action, begin):
    try:
        import version
        version = version.version
    except:
        version = "0";

    try:
        proxy.begin_action(
            ['svcname',
             'action',
             'hostname',
             'hostid',
             'version',
             'begin',],
            [repr(svc.svcname),
             repr(action),
             repr(rcEnv.nodename),
             repr(hostid),
             repr(version),
             repr(str(begin))]
        )
    except:
        pass

def end_action(svc, action, begin, end, logfile):
    err = 'ok'
    dateprev = None
    lines = open(logfile, 'r').read()

    """ If logfile is empty, default to current process pid
    """
    pid = os.getpid()

    """Example logfile line:
    2009-11-11 01:03:25,252;DISK.VG;INFO;unxtstsvc01_data is already up;10200;EOL
    """
    for line in lines.split(';EOL\n'):
        if line.count(';') != 4:
            continue
        date = line.split(';')[0]

        """Push to database the previous line, so that begin and end
        date are available.
        """
        if dateprev is not None:
            res = res.lower()
            res = res.replace(svc.svcname+'.','')
            try:
                proxy.res_action(
                    ['svcname',
                     'action',
                     'hostname',
                     'hostid',
                     'pid',
                     'begin',
                     'end',
                     'status_log',
                     'status'],
                    [repr(svc.svcname),
                     repr(res+' '+action),
                     repr(rcEnv.nodename),
                     repr(hostid),
                     repr(pid),
                     repr(str(dateprev)),
                     repr(str(date)),
                     repr(str(msg)),
                     repr(str(res_err))]
                )
            except:
                pass

        res_err = 'ok'
        (date, res, lvl, msg, pid) = line.split(';')
        if lvl is None or lvl == 'DEBUG':
            continue
        if lvl == 'ERROR':
            err = 'err'
            res_err = 'err'
        if lvl == 'WARNING' and err != 'err':
            err = 'warn'
        if lvl == 'WARNING' and res_err != 'err':
            res_err = 'warn'
        dateprev = date

    """Push the last log entry, using 'end' as end date
    """
    try:
        res = res.lower()
        res = res.replace(svc.svcname+'.','')
        proxy.res_action(
            ['svcname',
             'action',
             'hostname',
             'hostid',
             'pid',
             'begin',
             'end',
             'status_log',
             'status'],
            [repr(svc.svcname),
             repr(res+' '+action),
             repr(rcEnv.nodename),
             repr(hostid),
             repr(pid),
             repr(str(dateprev)),
             repr(str(end)),
             repr(str(msg)),
             repr(str(res_err))]
        )
    except:
        pass

    """Complete the wrap-up database entry
    """
    try:
        proxy.end_action(
            ['svcname',
             'action',
             'hostname',
             'hostid',
             'pid',
             'begin',
             'end',
             'time',
             'status'],
            [repr(svc.svcname),
             repr(action),
             repr(rcEnv.nodename),
             repr(hostid),
             repr(pid),
             repr(str(begin)),
             repr(str(end)),
             repr(str(end-begin)),
             repr(str(err))]
        )
    except:
        pass

def svcmon_update(svc, status):
    try:
        vars = [\
            "mon_svcname",
            "mon_svctype",
            "mon_nodname",
            "mon_nodtype",
            "mon_hostid",
            "mon_ipstatus",
            "mon_diskstatus",
            "mon_syncstatus",
            "mon_containerstatus",
            "mon_fsstatus",
            "mon_overallstatus",
            "mon_updated",
            "mon_prinodes"]
        vals = [\
            repr(svc.svcname),
            repr(svc.svctype),
            repr(rcEnv.nodename),
            repr(rcEnv.host_mode),
            repr(hostid),
            repr(str(status["ip"])),
            repr(str(status["disk"])),
            repr(str(status["sync"])),
            repr(str(status["container"])),
            repr(str(status["fs"])),
            repr(str(status["overall"])),
            repr(str(datetime.now())),
            repr(' '.join(svc.nodes))]
        proxy.svcmon_update(vars, vals)
    except:
        pass

def push_ips(svc):
    proxy.delete_ips(svc.svcname, rcEnv.nodename)
    for rset in svc.get_res_sets("ip"):
        for r in rset.resources:
            proxy.register_ip(
                ['ip_svcname',
                 'ip_dev',
                 'ip_name',
                 'ip_node',
                 'ip_netmask'],
                [repr(svc.svcname),
                 repr(r.ipDev),
                 repr(r.ipName),
                 repr(rcEnv.nodename),
                 repr(str(r.mask))]
            )

def push_fss(svc):
    proxy.delete_fss(svc.svcname)
    for rset in svc.get_res_sets("fs"):
        for r in rset.resources:
            proxy.register_fs(
                ['fs_svcname',
                 'fs_dev',
                 'fs_mnt',
                 'fs_mntopt',
                 'fs_type'],
                [repr(svc.svcname),
                 repr(r.device),
                 repr(r.mountPoint),
                 repr(r.mntOpt),
                 repr(r.fsType)]
            )

def push_rsyncs(svc):
    proxy.delete_fss(svc.svcname)
    for rset in svc.get_res_sets("fs"):
        for r in rset.resources:
            proxy.register_fs(
                ['fs_svcname',
                 'fs_dev',
                 'fs_mnt',
                 'fs_mntopt',
                 'fs_type'],
                [repr(svc.svcname),
                 repr(r.device),
                 repr(r.mountPoint),
                 repr(r.mntOpt),
                 repr(r.fsType)]
            )

def push_service(svc):
    def envfile(svc):
        envfile = os.path.join(rcEnv.pathsvc, 'etc', svc+'.env')
        if not os.path.exists(envfile):
            return
        with open(envfile, 'r') as f:
            buff = f.read()
            return buff
        return

    try:
        import version
        version = version.version
    except:
        version = "0";

    proxy.update_service(
        ['svc_hostid',
         'svc_name',
         'svc_vmname',
         'svc_type',
         'svc_nodes',
         'svc_drpnode',
         'svc_drpnodes',
         'svc_comment',
         'svc_drptype',
         'svc_autostart',
         'svc_app',
         'svc_containertype',
         'svc_envfile',
         'svc_version',
         'svc_drnoaction'],
        [repr(hostid),
         repr(svc.svcname),
         repr(svc.vmname),
         repr(svc.svctype),
         repr(' '.join(svc.nodes)),
         repr(svc.drpnode),
         repr(' '.join(svc.drpnodes)),
         repr(svc.comment),
         repr(svc.drp_type),
         repr(svc.autostart_node),
         repr(svc.app),
         repr(svc.svcmode),
         repr(envfile(svc.svcname)),
         repr(version),
         repr(svc.drnoaction)]
    )

def delete_services():
    proxy.delete_services(hostid)

def push_disks(svc):
    def disk_dg(dev, svc):
        for rset in svc.get_res_sets("disk.vg"):
            for vg in rset.resources:
                if vg.is_disabled():
                    continue
                if not vg.name in disklist_cache:
                    disklist_cache[vg.name] = vg.disklist()
                if dev in disklist_cache[vg.name]:
                    return vg.name
        return

    di = __import__('rcDiskInfo'+sysname)
    disks = di.diskInfo()
    disklist_cache = {}

    proxy.delete_disks(svc.svcname, rcEnv.nodename)

    for d in svc.disklist():
        proxy.register_disk(
            ['disk_id',
             'disk_svcname',
             'disk_size',
             'disk_vendor',
             'disk_model',
             'disk_dg',
             'disk_nodename'],
            [repr(disks.disk_id(d)),
             repr(svc.svcname),
             repr(disks.disk_size(d)),
             repr(disks.disk_vendor(d)),
             repr(disks.disk_model(d)),
             repr(str(disk_dg(d, svc))),
             repr(rcEnv.nodename)]
        )

def push_all(svcs):
    proxy.delete_service_list([svc.svcname for svc in svcs])
    for svc in svcs:
        push_rsyncs(svc)
        push_ips(svc)
        push_fss(svc)
        push_disks(svc)
        push_service(svc)

